/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.user

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.partner.user.model.AgeAuthInfo
import com.kakao.sdk.partner.user.model.PartnerUser
import com.kakao.sdk.partner.user.network.signupKapiWithOAuth
import com.kakao.sdk.user.UserApiClient

/**
 * @suppress
 */


val UserApiClient.partnerApi: PartnerUserApi
    get() = ApiFactory.kapiWithOAuth.create(PartnerUserApi::class.java)

val UserApiClient.partnerSignupApi: PartnerUserApi
    get() = ApiFactory.signupKapiWithOAuth.create(PartnerUserApi::class.java)


/**
 * 파트너용 /v2/user/me API 를 호출. 사용자에 대한 다양한 정보 획득.
 */
@JvmOverloads
fun UserApiClient.meForPartner(
    propertyKeys: List<String>? = null,
    secureReSource: Boolean = true,
    callback: (user: PartnerUser?, error: Throwable?) -> Unit
) {
    partnerApi.me(if (propertyKeys == null) null else KakaoJson.toJson(propertyKeys), secureReSource)
        .enqueue(object: ApiCallback<PartnerUser>() {
            override fun onComplete(model: PartnerUser?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 앱 연결 상태가 **PREREGISTER** 상태의 사용자에 대하여 앱 연결 요청. **자동연결** 설정을 비활성화한 앱에서 사용.
 */
@JvmOverloads
fun UserApiClient.signup(
    properties: Map<String, String>? = null,
    callback: (error: Throwable?) -> Unit
) {
    partnerSignupApi.signup(properties)
        .enqueue(object: ApiCallback<Unit>() {
            override fun onComplete(model: Unit?, error: Throwable?) {
                callback(error)
            }
        })
}

/**
 * 연령인증이 필요한 시점 연령인증 결과 조회를 확인 하기 위해 사용.
 *
 * @param ageLimit     reponse의 bypass_age_limit를 판단하는 기준 제한 연령
 * @param propertyKeys 추가 동의를 필요로 하는 인증 정보를 응답에 포함하고 싶은 경우, 해당 키 리스트
 * @param callback     연령인증 조회 결과 [AgeAuthInfo] 수신
 *
 */
@JvmOverloads
fun UserApiClient.ageAuthInfo(
    ageLimit: Int? = null ,
    propertyKeys: List<String>? = null,
    callback: (info : AgeAuthInfo?, error: Throwable?) -> Unit
) {
    partnerApi.ageAuthInfo(ageLimit, if (propertyKeys == null) null else KakaoJson.toJson(propertyKeys))
        .enqueue(object: ApiCallback<AgeAuthInfo>() {
            override fun onComplete(model: AgeAuthInfo?, error: Throwable?) {
                callback(model, error)
            }
        })
}
